
//---------------------------------------------------------------------------

// This software is Copyright (c) 2011 Embarcadero Technologies, Inc. 
// You may only use this software if you are an authorized licensee
// of Delphi, C++Builder or RAD Studio (Embarcadero Products).
// This software is considered a Redistributable as defined under
// the software license agreement that comes with the Embarcadero Products
// and is subject to that software license agreement.

//---------------------------------------------------------------------------
unit FishFactsServerMethods;

interface

uses
  SysUtils, Classes, DSServer, DB, DBTables, DBXCommon, DBXDBReaders, DBXJSon,
  DBClient, DataModule;

type
{$METHODINFO ON}
  TFishFactsServerMethods = class(TComponent)
  private
    FDataModule: TFactsDataModule;
    procedure GetKeys(const Container: TJSONArray); overload;
    function DataModule: TFactsDataModule;
  public
    destructor Destroy; override;
    function ReverseString(Value: string): string;
    function GetKeys: TJSONArray; overload;
    function GetImage(AKey: String): TStream;
    function GetFacts(const AKey: string;
      out ASpeciesName: string;
      out ACategory: string;
      out ACommonName: string;
      out ALengthIn: Double;
      out ALengthCm: Double): Boolean;
    function GetNotes(const AKey: string;
      out ANotes: string): Boolean;
  end;
{$METHODINFO OFF}

implementation

uses StrUtils, HTTPApp, Types, Graphics, Jpeg, Datasnap.DSService;

function TFishFactsServerMethods.DataModule: TFactsDataModule;
const
  cFactsDataModule = 'FactsDataModule';
begin
  if FDataModule = nil then
    FDataModule := TFactsDataModule.Create(nil);
  result := FDataModule;
end;

destructor TFishFactsServerMethods.Destroy;
begin
  FreeAndNil(FDataModule);
  inherited;
end;

function TFishFactsServerMethods.GetFacts(const AKey: string;
  out ASpeciesName: string;
  out ACategory: string;
  out ACommonName: string;
  out ALengthIn: Double;
  out ALengthCm: Double): Boolean;
begin
  ASpeciesName := '';
  ACategory := '';
  ACommonName := '';
  ALengthIn := 0;
  ALengthCm := 0;
  DataModule.qryRecord.ParamByName('Key').AsString := AKey;
  DataModule.qryRecord.Active := True;
  try
    if not DataModule.qryRecord.Eof then begin
      ASpeciesName := DataModule.qryRecordSPECIESNAME.AsString;
      ACategory := DataModule.qryRecordCATEGORY.AsString;
      ACommonName := DataModule.qryRecordCOMMON_NAME.AsString;
      ALengthIn := DataModule.qryRecordLENGTH_IN.AsFloat;
      ALengthCm := DataModule.qryRecordLENGTHCM.AsFloat;
      Result := True;
    end
    else
    begin
      Result := False;
    end;
  finally
     DataModule.qryRecord.Active := False;
  end;
end;

function TFishFactsServerMethods.GetImage(AKey: String): TStream;
var
  LStream: TStream;
  LPicture: TPicture;
  LJPG: TJPEGImage;
begin
  Result := nil;
  DataModule.qryRecord.ParamByName('Key').AsString := AKey;
  DataModule.qryRecord.Active := True;
  try
    if not DataModule.qryRecord.EoF then begin
      LStream := TMemoryStream.Create;
      try
        LPicture := TPicture.Create;
        try
          LPicture.Assign(DataModule.qryRecordGRAPHIC);
          LJPG := TJPEGImage.Create;
          try
            LJPG.Assign(LPicture.Graphic);
            LJPG.SaveToStream(LStream);
          finally
            LJPG.Free;
          end;
        finally
          LPicture.Free;
        end;
        LStream.Position := 0;
        Result := LStream;
        Exit;
      except
        FreeAndNil(LStream);
        raise;
      end;
    end;
  finally
    DataModule.qryRecord.Active := False;
  end;
end;

function TFishFactsServerMethods.GetKeys: TJSONArray;
var
  Container: TJSONArray;
begin
  Container := TJSONArray.Create;
  GetKeys(Container);
  Result := Container;
end;

function TFishFactsServerMethods.GetNotes(const AKey: string;
  out ANotes: string): Boolean;
begin
  ANotes := '';
  DataModule.qryRecord.ParamByName('Key').AsString := AKey;
  DataModule.qryRecord.Active := True;
  try
    if not DataModule.qryRecord.Eof then begin
      ANotes := DataModule.qryRecordNOTES.AsString;
      Result := True;
    end
    else
    begin
      Result := False;
    end;
  finally
     DataModule.qryRecord.Active := False;
  end;
end;

procedure TFishFactsServerMethods.GetKeys(const Container: TJSONArray);
begin
  DataModule.qryKeys.Active := True;
  try
    DataModule.qryKeys.First;
    while not DataModule.qryKeys.Eof do
    begin
      Container.AddElement(TJSONString.Create(DataModule.qryKeysSPECIES_NO.AsString));
      DataModule.qryKeys.Next;
    end;
  finally
    DataModule.qryKeys.Active := False;
  end;
end;

function TFishFactsServerMethods.ReverseString(Value: string): string;
begin
  Result := StrUtils.ReverseString(Value);
end;

end.

